from .GuessGame import guess_game
from .CurrencyRouletteGame import Currency_Roulette_Game
from .MemoryGame import memory_game
from .Common import common
from .Score import Score

class game_manager(common):
  def __init__(self):
      self.games = []
      self.text_menu = ''
      self.create_games_obj()
      self.create_menu()
      self.game_score = Score()

  # need to create all object to get the menu
  def create_games_obj(self):
    new_game = guess_game()
    self.games.append(new_game)
    new_game = memory_game()
    self.games.append(new_game)
    new_game = Currency_Roulette_Game('Currency Roulette - try and guess the value of a random amount of USD in ILS')
    self.games.append(new_game)

  def create_menu(self):
    self.text_menu = "Please choose a game to play:\n"
    index = 0
    for games in self.games:
      index += 1
      new_line = index.__str__() + '. ' + games.get_menu() + '\n'
      self.text_menu += new_line

  def play_game(self, game_obj, difficulty):
    game_result = game_obj.play(difficulty=difficulty)
    if game_result:
      self.game_score.add_score(difficulty)
    print(game_result)

  def show_main_menu(self):
    self.clean_screen()
    game_number = self.start_validate_input(message=self.text_menu, val_from_value=1, val_to_value=3)
    difficulty_number = self.start_validate_input("Please choose game difficulty from 1 to 5: ", val_from_value=1, val_to_value=5)
    self.clean_screen()
    self.play_game(self.games[game_number-1], difficulty_number)

  def welcome(self, name):
    return "Hello " + name + " and welcome to the World of Games(WoG).\nHere you can find many cool games to play."


from flask import Flask, render_template
import .Score

game_score = Score.Score()
game_score.add_score(2)

app = Flask(__name__)

@app.route('/')


def homepage():
    current_score = game_score.read_score()
    if current_score == -1:
      return render_template('error.html', ERROR=current_score)
    else:
      return render_template('index.html', SCORE=current_score)

if __name__ == '__main__':
    app.run()
else:
    print('Please run this file as main, not as module.')



import os

class common():

  def get_int_value(self):
    while True:
      value = input()
      if str(value).isdigit():
        return int(value)
      else:
        print('Please enter only numeric values!')

  def get_int_or_float_value(self):
    while True:
      value = input()
      if type(value == float):
        try:
          return float(value)  # in case user will type "33.4333fseffdd"
        except:
          print('Please enter only numeric')
      elif type(value == int):
        return int(value)
      else:
        print('Please enter only numeric')

  def is_corret_value(self, input_num, from_value, to_value):
    if (input_num >= from_value) and (input_num <= to_value):
      return True
    else:
      return False

  def is_big_value(self, input_num, to_value):
    if (input_num > to_value):
      return True

  def start_validate_input(self, message, val_from_value, val_to_value):
    print(message)
    answer_correct = False
    while not answer_correct:
      user_input_number = self.get_int_value()
      if self.is_corret_value(input_num=user_input_number, from_value=val_from_value, to_value=val_to_value):
        answer_correct = True
      else:
        if self.is_big_value(input_num=user_input_number, to_value=val_to_value):
          print(f'{user_input_number} is too big, ' + message)
        else:
          print(f'{user_input_number} is too small, ' + message)

    return user_input_number

  def clean_screen(self):
    os.system("cls")


from .Const import *
import os.path
import json

# This class is in charge of managing the scores file.
class Score():
  def __init__(self):
      self.score_file_name = SCORES_FILE_NAME

  def read_score(self):
      if os.path.isfile(self.score_file_name):
        try:
          score_file = open(self.score_file_name)
          dictionary = json.load(score_file)
          score_value = dictionary.get(SCORES_LABEL, 0)
          score_file.close()
          return score_value
        except:
          return BAD_RETURN_CODE
      else:
        return 0

  def add_score(self, difficulty):
      current_score = self.read_score() # score from the file
      if current_score >= 0: # in case of good reading from the file
        new_score = current_score + self.calc_new_score(difficulty)
      else: # in case of bad reading from the file or no file
        new_score = self.calc_new_score(difficulty)

      # Save score in dict to the file
      dictionary = {SCORES_LABEL: new_score} # score value
      try: # file handle
          score_file = open(self.score_file_name, 'wt')
          json.dump(dictionary, score_file)  # save as JSON
          score_file.close()
          return GOOD_RETURN_CODE
      except:
          return BAD_RETURN_CODE

  def calc_new_score(self, difficulty):
      return (difficulty * 3) + 5




from abc import ABC, abstractmethod
from .Common import common
from random import *

class game(ABC, common):
  def __init__(self, override_menu):
    self.difficulty = 1
    self.override_menu = override_menu

  @abstractmethod
  def play(self, difficulty):
    pass

  @abstractmethod
  def get_menu(self):
    pass

  def generate_number(self, last_number):
    if self.difficulty == 1:
      return 1
    else:
      return randint(1, last_number)

  def get_guess_from_user(self, val_from_value, val_to_value):
    return self.start_validate_input(f'Please guess a number between {val_from_value} and {val_to_value}:', val_from_value = val_from_value, val_to_value = val_to_value)

  def compare_results(self,master_number, val_from_value, val_to_value):
    return (master_number == self.get_guess_from_user(val_from_value= val_from_value, val_to_value= val_to_value))



from .Game import game
from exchangeratesapi import Api


class Currency_Roulette_Game(game):
  def __init__(self, override_menu=''):
      self.override_menu = override_menu


  def play(self, difficulty):
      self.difficulty = difficulty
      from_int, to_int = self._get_money_interval()
      return self._compare_results(master_number= self._get_guess_from_user(), val_from_value= from_int, val_to_value= to_int)

  def get_menu(self):
      if self.override_menu:
        return self.override_menu
      else:
        return "Currency Roulette - try and guess the value of a random amount of USD in ILS."

  def _get_rate(self):
     api = Api()
     return  api.get_rate('USD', 'ILS')

  def _get_money_interval(self):
      t = (self._get_rate() * self.generate_number(last_number = 100))
      d = self.difficulty
      return (t - (5 - d), t +  (5 - d))

  def _get_guess_from_user(self):
      print('Please guess what will be the generated number from USD to ILS:')
      return self.get_int_or_float_value()

  def _compare_results(self,master_number, val_from_value, val_to_value):
    return self.is_corret_value(input_num= master_number, from_value= val_from_value, to_value= val_to_value)


from setuptools import setup, find_packages

# specify requirements of your package here
REQUIREMENTS = ['python-exchangeratesapi', 'flask', 'wheel']

# some more details
CLASSIFIERS = [
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'Topic :: Internet',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.8'
    ]

# calling the setup function 
setup(name='eliwog',
      version='4.0.0',
      description='DevOps course project',
      long_description='This package will install my devops course project. use python: import eliwog',
      url='https://github.com/EliGolan69/WorldOfGames.git',
      author='Eli Golan',
      author_email='1eligolan@gmail.com',
      license='MIT',
      packages=['eliwog'],
      classifiers=CLASSIFIERS,
      install_requires=REQUIREMENTS,
      keywords='DevOps',
      include_package_data = True
      )



from .Game import game


class guess_game(game):
  def __init__(self, override_menu=''):
      self.override_menu = override_menu

  def play(self, difficulty):
    self.difficulty = difficulty
    self.secret_number = self.generate_number(last_number= difficulty)
    return self.compare_results(master_number= self.secret_number, val_from_value= 1, val_to_value= difficulty)

  def get_menu(self):
      if self.override_menu:
        return self.override_menu
      else:
        return "Guess Game - guess a number and see if you chose like the computer."



from random import *
from .Game import game
from abc import ABC, abstractmethod
import time

class memory_game(game):
  def __init__(self, override_menu=''):
      self.override_menu = override_menu

  def play(self, difficulty):
    self.difficulty = difficulty
    return self._is_list_equal(game_list = self._generate_sequence(), user_list = self._get_list_from_user())

  def get_menu(self):
      if self.override_menu:
        return self.override_menu
      else:
        return "Memory Game - a sequence of numbers will appear for 1 second and you have to guess it back."

  def _generate_sequence(self):
    secret_number = []
    for index in range(self.difficulty):
      secret_number.append(randrange(1, 101))

    print(secret_number)
    time.sleep(0.7)
    self.clean_screen()
    return secret_number

  def _get_list_from_user(self):
    user_enter_numbers = False
    user_number_list = []
    print('\n\nPlease try to guess the numbers.')
    while not user_enter_numbers:
      print('Please enter a number: ')
      user_number_list.append(self.get_int_value())
      if len(user_number_list) == self.difficulty:
        user_enter_numbers = True

    return user_number_list

  def _is_list_equal(self, game_list, user_list):
    list_equal = True
    for number in game_list:
      if user_list.count(number) <= 0:
        list_equal = False
        break

    return list_equal








#  Scores file name
SCORES_FILE_NAME = 'Scores.txt'
# score label in the file
SCORES_LABEL = "score"
# A number representing a bad return code for a function.
BAD_RETURN_CODE = -1

GOOD_RETURN_CODE = 0